<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\Client\Request;

use ConnectPayPayment\Components\Struct\Configuration;
use Shopware\Core\Framework\Struct\ExtendableInterface;
use \Shopware\Core\Framework\Struct\CloneTrait;
use \Shopware\Core\Framework\Struct\JsonSerializableTrait;
use \Shopware\Core\Framework\Struct\AssignArrayTrait;
use \Shopware\Core\Framework\Struct\ExtendableTrait;
use \Shopware\Core\Framework\Struct\CreateFromTrait;
use \Shopware\Core\Framework\Struct\VariablesAccessTrait;

abstract class Struct implements \JsonSerializable, ExtendableInterface
{
    public const USERNAME = 'clientId';
    public const PASSWORD = 'clientSecret';
    
    //allows to clone full struct with all references
    use CloneTrait;

    //allows json_encode and to decode object via json serializer
    use JsonSerializableTrait;

    //allows to assign array data to this object
    use AssignArrayTrait;

    //allows to add values to an internal attribute storage
    use ExtendableTrait;

    //allows to create a new instance with all data of the provided object
    use CreateFromTrait;

    //allows access to all protected variables of the object
    use VariablesAccessTrait;
    
    protected Configuration $configuration;
    
    public function __construct(Configuration $configuration) {
        $this->configuration = $configuration;
    }

    public function getApiAlias(): string
    {
        $class = static::class;

        $class = explode('\\', $class);
        $class = implode('', $class);

        return ltrim(mb_strtolower(preg_replace('/[A-Z]/', '_$0', $class)), '_');
    }
    
    public function getBaseUrl(): string
    {
        if ($this->configuration->get('prodTestSwitcher')) {
            return static::BASE_PROD_URL;
        } else {
            return static::BASE_STAGE_URL;
        }
    }     
    
    public function getBasicAuth(): string
    {
        return 'Basic ' . base64_encode(
            sprintf('%s:%s', $this->configuration->get(self::USERNAME), $this->configuration->get(self::PASSWORD))
        );
    }
}
