<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\Service\ConfigService;

use ConnectPayPayment\Components\Struct\Configuration;
use Shopware\Core\Framework\Context;

interface ConfigServiceInterface
{
    public const BASE_PATH = 'ConnectPayPayment';
    public const CONFIG_PATH = 'config';
    public const CERTIFICATE_PATH = 'config/connect-pay/';
    public const PEM_KEY = 'pemAuth';
    public const PROVIDERS = 'ConnectPayPayment.config.providersSettingsProvider';

    public const BRAND_ID = 'brandId';
    public const PROVIDER_COUNTRY_CODE = 'providersSettingsCountry';
    public const MERCHANT_LANGUAGE = 'merchantLanguage';
    public const DEBUG_MODE = 'debug';

    /**
     * Get plugin configuration
     *
     * @param string|null $salesChannelId
     * @return Configuration
     */
    public function get(?string $salesChannelId = null): Configuration;

    /**
     * Set new config value
     *
     * @param string $key
     * @param $value
     * @param string $path
     * @param string|null $salesChannelId
     * @return void
     */
    public function set(string $key, $value, string $path, ?string $salesChannelId = null): void;

    /**
     * Remove config value
     *
     * @param string $key
     * @param string $path
     * @param string|null $salesChannelId
     * @return void
     */
    public function unset(string $key, string $path, ?string $salesChannelId = null): void;

    /**
     * Check plugin status
     *
     * @param Context $context
     * @return bool
     */
    public function pluginActive(Context $context): bool;
}
