import template from './connectpay-secure-textarea.html.twig';
import './connectpay-secure-textarea.scss';

const { Component, Mixin } = Shopware;

/**
 * @description secure textarea input field.
 * @status ready
 * @example-type static
 * @component-example
 * <connectpay-secure-textarea type="textarea" label="Name" placeholder="placeholder goes here..."></connectpay-secure-textarea>
 */
Component.register('connectpay-secure-textarea', {
    template,
    inheritAttrs: false,

    mixins: [
        Mixin.getByName('sw-form-field'),
        Mixin.getByName('remove-api-error'),
    ],

    props: {
        value: {
            type: String,
            required: false,
            default: null,
        },

        placeholder: {
            type: String,
            required: false,
            default: null,
        },
    },

    data() {
        return {
            currentValue: this.value.replace(/./g, '*') || '',
        };
    },

    watch: {
        value() { this.currentValue = this.value; },
    },

    methods: {
        onInput(event) {
            this.$emit('input', event.target.value);
        },

        onChange(event) {
            this.$emit('change', event.target.value);
        },
    },
});
