<?php declare(strict_types=1);

namespace ConnectPayPayment\ScheduledTask;

use ConnectPayPayment\Components\Helper\OrderTransaction\OrderTransactionHelperInterface;
use ConnectPayPayment\Components\PaymentHandler\AbstractPaymentHandler;
use Psr\Log\LoggerInterface;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\MessageQueue\ScheduledTask\ScheduledTaskHandler;

class GetPaymentDetailsHandler extends ScheduledTaskHandler
{
    private AbstractPaymentHandler $paymentHandler;
    private OrderTransactionHelperInterface $orderTransactionHelper;
    private LoggerInterface $logger;

    /**
     * @param EntityRepositoryInterface $scheduledTaskRepository
     * @param AbstractPaymentHandler $paymentHandler
     * @param OrderTransactionHelperInterface $orderTransactionHelper
     * @param LoggerInterface $logger
     */
    public function __construct(
        EntityRepositoryInterface $scheduledTaskRepository,
        AbstractPaymentHandler $paymentHandler,
        OrderTransactionHelperInterface $orderTransactionHelper,
        LoggerInterface $logger
    ) {
        $this->paymentHandler = $paymentHandler;
        $this->orderTransactionHelper = $orderTransactionHelper;
        $this->logger = $logger;

        parent::__construct($scheduledTaskRepository);
    }

    /**
     * {@inheritdoc}
     */
    public static function getHandledMessages(): iterable
    {
        return [GetPaymentDetails::class];
    }

    public function run(): void
    {
        foreach ($this->orderTransactionHelper->getConnectPayTransactions() as $orderTransaction) {
            if (!isset($orderTransaction->getCustomFields()['paymentId'])) {
                continue;
            }

            $response = $this->paymentHandler->checkPaymentStatus($orderTransaction);

            if (!$response) {
                $this->logger->debug('Something went wrong during retrieving payment details by ConnectPay');

                return;
            }

            $responseDetails = $response->getResponseDetails();

            $this->logger->debug(
                $responseDetails['payment']['identifiers']['merchantReference'].' transaction payment was processed by ConnectPay'
            );
        }
    }
}
