<?php declare(strict_types=1);

namespace ConnectPayPayment\Subscriber;

use ConnectPayPayment\Components\Client\Hydrator\Request\InitiatePayment\InitiatePaymentRequestHydratorInterface;
use ConnectPayPayment\Components\Service\ConfigService\ConfigServiceInterface;
use ConnectPayPayment\Installer\PaymentInstaller;
use Shopware\Storefront\Page\Checkout\Confirm\CheckoutConfirmPageLoadedEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsAnyFilter;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\Search\EntitySearchResult;

class CheckoutConfirmPageSubscriber implements EventSubscriberInterface
{
    private ConfigServiceInterface $configService;
    private EntityRepository $providersRepository;  

    /**
     * @param ConfigServiceInterface $configService
     * @param EntityRepository $providersRepository
     */
    public function __construct(
        ConfigServiceInterface $configService,
        EntityRepository $providersRepository
    ) {
        $this->configService = $configService;
        $this->providersRepository = $providersRepository;
    }

    /**
     * {@inheritDoc}
     */
    public static function getSubscribedEvents(): array
    {
        return [
            CheckoutConfirmPageLoadedEvent::class => 'onCheckoutConfirmPageLoaded',
        ];
    }

    /**
     * Filter payment methods
     *
     * @param CheckoutConfirmPageLoadedEvent $event
     */
    public function onCheckoutConfirmPageLoaded(CheckoutConfirmPageLoadedEvent $event): void
    {
        setcookie('bank_provider', '', 0, '/');
        $currencyCode = $event->getSalesChannelContext()->getCurrency()->getIsoCode();

        $payments = array_keys(PaymentInstaller::PAYMENT_METHODS);

        if (!($this->configService->pluginActive($event->getSalesChannelContext()->getContext())
            && in_array($currencyCode, InitiatePaymentRequestHydratorInterface::PAYMENT_METHOD)
        )) {
            foreach ($payments as $payment) {
                $event->getPage()->getPaymentMethods()->filterAndReduceByProperty('handlerIdentifier', $payment);
            }
        }
        
        $event->getPage()->assign([
            'providers' => $this->getProvidersData($event->getSalesChannelContext()->getContext())
        ]);
    }
    
    /**
     * @param Context $context
     * @return EntitySearchResult|null
     */
    private function getProvidersData(Context $context): ?EntitySearchResult
    {
        $providersIds = $this->configService->get()->get('providersSettingsProvider');

        $criteria = new Criteria();

        if ($providersIds) {
            $criteria->addFilter(
                new EqualsAnyFilter('id', explode('|', $providersIds))
            );
            
            return $this->providersRepository->search($criteria, $context);
        }
        
        return null;
    }
}
